@@ -304,7 +304,11 @@ class TimeSeries(np.ndarray):
             Index such that abs(self.time[idx]-t) is as small as possible
 
         """
-        return np.argmin(np.abs(self.t - t))
+        idx = min(np.searchsorted(self.t, t), self.n_times-1)
+        if idx > 0:
+            if abs(self.t[idx]-t) > abs(self.t[idx-1]-t):
+                idx -= 1
+        return idx
 
     @property
     def n_times(self):
@@ -560,6 +564,7 @@ class TimeSeries(np.ndarray):
         if callable(abs_tolerance):
             abs_tolerance = abs_tolerance(self)
         power_of_2 = (2.0 ** np.floor(-np.log2(abs_tolerance)))
-        self.ndarray *= power_of_2
-        np.round(self.ndarray, out=self.ndarray)
-        self.ndarray /= power_of_2
+        ndarray = self.ndarray
+        ndarray *= power_of_2
+        np.round(ndarray, out=ndarray)
+        ndarray /= power_of_2
