@@ -91,8 +91,8 @@ class WaveformModes(WaveformMixin, TimeSeries):
             if key == "History.txt":
                 return self._metadata.get("history", "")
             # Assume we're asking for Y_l2_m2.dat or something
-            if self.data.shape != (self.n_times, self.n_modes):
-                raise ValueError(f"Data has shape {self.data.shape}, which is incompatible with NRAR format")
+            if self.ndarray.shape != (self.n_times, self.n_modes):
+                raise ValueError(f"Data has shape {self.ndarray.shape}, which is incompatible with NRAR format")
             match = NRAR_mode_regex.match(key)
             if not match:
                 raise ValueError(f"Key '{key}' did not match mode format")
@@ -102,7 +102,7 @@ class WaveformModes(WaveformMixin, TimeSeries):
             if abs(m) > ell:
                 raise ValueError(f"Key '{key}' requested (ell, m)=({ell}, {m}) value does not make sense")
             index = self.index(ell, m)
-            data = np.take(self.data, index, axis=self.modes_axis).view(float).reshape((-1, 2))
+            data = np.take(self.ndarray, index, axis=self.modes_axis).view(float).reshape((-1, 2))
             return np.hstack((self.time[:, np.newaxis], data))
         obj, time_key = self._slice(key)
         if time_key is None:
@@ -170,7 +170,7 @@ class WaveformModes(WaveformMixin, TimeSeries):
             Index such that self.LM[idx] == [ell, m]
 
         """
-        return spherical.LM_index(ell, m, self.ell_min)
+        return spherical.Yindex(ell, m, self.ell_min)
 
     @property
     def abs(self):
@@ -284,7 +284,7 @@ class WaveformModes(WaveformMixin, TimeSeries):
             conjugate(f){s, l, m} = (-1)**(s+m) * conjugate(f{-s, l, -m})
 
         """
-        return spherical.SWSH_modes.algebra.bar(self)
+        return spherical.modes.algebra.bar(self)
 
     @property
     def re(self):
@@ -321,7 +321,7 @@ class WaveformModes(WaveformMixin, TimeSeries):
             f{l, m} = (f{l, m} + (-1)**m * conjugate(f{l, -m})) / 2
 
         """
-        return spherical.SWSH_modes.algebra._real_func(self, False)
+        return spherical.modes.algebra._real_func(self, False)
 
     @property
     def im(self):
@@ -359,9 +359,9 @@ class WaveformModes(WaveformMixin, TimeSeries):
             f{l, m} = (f{l, m} + (-1)**(m+1) * conjugate(f{l, -m})) / 2
 
         """
-        return spherical.SWSH_modes.algebra._imag_func(self, False)
+        return spherical.modes.algebra._imag_func(self, False)
 
-    from spherical.SWSH_modes.derivatives import (
+    from spherical.modes.derivatives import (
         Lsquared, Lz, Lplus, Lminus,
         Rsquared, Rz, Rplus, Rminus,
         eth, ethbar
@@ -603,7 +603,6 @@ class WaveformModes(WaveformMixin, TimeSeries):
         >>> w.evaluate(equiangular)
 
         """
-        import string
         if len(directions) == 1:
             directions = directions[0]
         if isinstance(directions, quaternionic.array):
@@ -612,7 +611,6 @@ class WaveformModes(WaveformMixin, TimeSeries):
             directions = np.asarray(directions, dtype=float)
             if directions.shape[-1] == 2:
                 R = quaternionic.array.from_spherical_coordinates(directions)
-                #R = quaternionic.array.from_spherical_coordinates(directions[..., 0], directions[..., 1])
             elif directions.shape[-1] == 3:
                 R = quaternionic.array.from_euler_angles(directions[..., 1], directions[..., 0], directions[..., 2])
             else:
@@ -620,31 +618,43 @@ class WaveformModes(WaveformMixin, TimeSeries):
                     f"Input `directions` array must be quaternionic, or "
                     f"final dimension of must have size 2 or 3, not {directions.shape[-1]}"
                 )
-        # R.shape == directions.shape[:-1] + (4,)
-        if self.frame == np.atleast_2d(quaternionic.one):
-            sYlm = spherical.SWSH_grid(R, self.spin_weight, self.ell_max)
-            sYlm = sYlm[..., spherical.LM_index(self.ell_min, -self.ell_min, 0):]  # Chop off leading zeros
-            # sYlm.shape == directions.shape[:-1] + (self.n_modes,)
-            result = np.tensordot(self.data, sYlm, axes=[[self.modes_axis], [-1]])
-        else:
-            # We have to account for a rotating frame
-            frame = self.frame.reshape((self.frame.shape[0],) + (1,)*(R.ndim-1) + (4,))
-            R = frame.inverse * R[np.newaxis]
-            sYlm = spherical.SWSH_grid(R, self.spin_weight, self.ell_max)
-            sYlm = sYlm[..., spherical.LM_index(self.ell_min, -self.ell_min, 0):]  # Chop off leading zeros
-            # sYlm.shape == frame.shape[:1] + directions.shape[:-1] + (self.n_modes,)
-            self_indices = list(string.ascii_letters[:self.ndim])
-            self_indices[self.time_axis] = "y"
-            self_indices[self.modes_axis] = "z"
-            self_indices = ''.join(self_indices)
-            sYlm_indices = list(string.ascii_letters[self.ndim:self.ndim+sYlm.ndim])
-            sYlm_indices[0] = "y"
-            sYlm_indices[-1] = "z"
-            sYlm_indices = ''.join(sYlm_indices)
-            result_indices = self_indices.replace("z", "") + sYlm_indices.replace("y", "").replace("z", "")
-            subscripts = f"{self_indices},{sYlm_indices}->{result_indices}"
-            result = np.einsum(subscripts, self.data, sYlm)
-        return TimeSeries(result, self.time)
+
+        # Compute the shape of the output
+        modes_axis = self.modes_axis
+        out_shape = (
+            self.shape[:modes_axis]
+            + R.shape[:-1]
+            + tuple() if (modes_axis % self.ndim) == (-1 % self.ndim) else self.shape[modes_axis+1:]
+        )
+
+        # For now, we'll keep the new dimensions flat
+        Rflat = R.ndarray.reshape(-1, 4)
+        signal_shape = list(self.shape)
+        signal_shape[modes_axis] = Rflat.shape[0]
+        signal = np.zeros(tuple(signal_shape), dtype=complex)
+
+        # Now, loop through, evaluating for each input R value
+        wigner = spherical.Wigner(self.ell_max, ell_min=self.ell_min, mp_max=abs(self.spin_weight))
+        sYlm = np.empty(wigner.Ysize, dtype=complex)
+        slices = [slice(None) for _ in range(signal.ndim)]
+        if np.array_equal(self.frame, np.atleast_2d(quaternionic.one)):  # frame is time-independent
+            for i_R in range(Rflat.shape[0]):
+                slices[modes_axis] = i_R
+                wigner.sYlm(self.spin_weight, Rflat[i_R], out=sYlm)
+                signal[tuple(slices)] = np.dot(self.ndarray, sYlm)
+        else:  # Need to account for time-dependent frame
+            data_slices = [slice(None) for _ in range(self.ndarray.ndim)]
+            time_axis = self.time_axis
+            for i_t in range(self.n_times):
+                slices[time_axis] = i_t
+                data_slices[time_axis] = i_t
+                R_t = self.frame[i_t].inverse * Rflat
+                for i_R, R_i in enumerate(Rflat):
+                    slices[modes_axis] = i_R
+                    wigner.sYlm(self.spin_weight, R_i, out=sYlm)
+                    signal[tuple(slices)] = np.dot(self.ndarray[tuple(data_slices)], sYlm)
+
+        return TimeSeries(signal.reshape(out_shape), self.time)
 
     # TODO:
     # expectation_value_LL
@@ -710,20 +720,56 @@ class WaveformModes(WaveformMixin, TimeSeries):
         Parameters
         ----------
         quat : quaternionic.array
-            This must have the same number of quaternions as the number of times in the
-            waveform.
+            This must have one quaternion or the same number of quaternions as the
+            number of times in the waveform.
 
         """
-        if quat.shape != (self.n_times, 4):
-            raise ValueError(f"Quaternionic array shape {quat.shape} not understood; expected {(self.n_times, 4)}")
-        D = np.empty((spherical.WignerD._total_size_D_matrices(self.ell_min, self.ell_max),), dtype=complex)
-        new_data = self.data.copy()
-        quat2spinor = quat.two_spinor
-        _rotate_decomposition_basis(new_data, quat2spinor.a, quat2spinor.b, self.ell_min, self.ell_max, D)
+        from spherical.wigner import _rotate
+
+        R = quaternionic.array(quat)
+        wigner = spherical.Wigner(self.ell_max, ell_min=self.ell_min)  #, mp_max=abs(self.spin_weight))
+        D = np.zeros(wigner.Dsize, dtype=complex)
+        mode_weights = self.ndarray
+        rotated_mode_weights = np.zeros_like(mode_weights)
+        mode_weights = np.moveaxis(mode_weights, self.modes_axis, -1)
+        rotated_mode_weights = np.moveaxis(rotated_mode_weights, self.modes_axis, -1)
+        shape = rotated_mode_weights.shape
+        if quat.shape == (4,) or quat.shape == (1, 4):
+            wigner.D(R, out=D)
+            mode_weights = mode_weights.reshape(-1, mode_weights.shape[-1])
+            rotated_mode_weights = rotated_mode_weights.reshape(-1, mode_weights.shape[-1])
+            _rotate(
+                mode_weights, rotated_mode_weights,
+                wigner.ell_min, wigner.ell_max, wigner.mp_max,
+                self.ell_min, self.ell_max, self.spin_weight,
+                D
+            )
+        elif quat.shape == (self.n_times, 4):
+            slices = [slice(None) for _ in range(self.ndim)]
+            time_axis = self.time_axis if self.time_axis < self.modes_axis else self.time_axis - 1
+            for i_t in range(self.n_times):
+                wigner.D(R[i_t], out=D)
+                slices[time_axis] = i_t
+                s = tuple(slices)
+                m = mode_weights[s]
+                r = rotated_mode_weights[s]
+                m = m.reshape(-1, m.shape[-1])
+                r = r.reshape(-1, r.shape[-1])
+                _rotate(
+                    m, r,
+                    wigner.ell_min, wigner.ell_max, wigner.mp_max,
+                    self.ell_min, self.ell_max, self.spin_weight,
+                    D
+                )
+        else:
+            raise ValueError(
+                f"Quaternionic array shape {R.shape} not understood; expected {(4,)}, {(1, 4)}, or {(self.n_times, 4)}"
+            )
+        rotated_mode_weights = rotated_mode_weights.reshape(shape)
+        rotated_mode_weights = np.moveaxis(rotated_mode_weights, -1, self.modes_axis)
         new_metadata = self._metadata.copy()
-        new_metadata.pop("frame")
-        w = type(self)(new_data, **new_metadata)
-        return w
+        new_metadata.pop("frame", None)
+        return type(self)(rotated_mode_weights, **new_metadata)
 
     def to_inertial_frame(self):
         """Return a copy of this waveform in the inertial frame"""
@@ -793,28 +839,3 @@ class WaveformModes(WaveformMixin, TimeSeries):
                 return (w, log_frame)
             else:
                 return w
-
-
-@jit("void(c16[:,:], c16[:], c16[:], i8, i8, c16[:])")
-def _rotate_decomposition_basis(data, R_basis_a, R_basis_b, ell_min, ell_max, D):
-    """Rotate data by a different rotor at each point in time
-
-    `D` is just a workspace, which holds the Wigner D matrices.
-    During the summation, it is also used as temporary storage to hold
-    the results for each item of data, where the first row in the
-    matrix is overwritten with the new sums.
-
-    """
-    for i_t in range(data.shape[0]):
-        spherical._Wigner_D_matrices(R_basis_a[i_t], R_basis_b[i_t], ell_min, ell_max, D)
-        for ell in range(ell_min, ell_max + 1):
-            i_data = ell ** 2 - ell_min ** 2
-            i_D = spherical._linear_matrix_offset(ell, ell_min)
-
-            for i_m in range(2 * ell + 1):
-                new_data_mp = 0j
-                for i_mp in range(2 * ell + 1):
-                    new_data_mp += data[i_t, i_data + i_mp] * D[i_D + i_m + (2 * ell + 1) * i_mp]
-                D[i_D + i_m] = new_data_mp
-            for i_m in range(2 * ell + 1):
-                data[i_t, i_data + i_m] = D[i_D + i_m]
