@@ -2,13 +2,21 @@ import contextlib
 import sys
 import pathlib
 import tempfile
+
 import numpy as np
+import quaternionic
+import spherical
 import h5py
 import pytest
 import sxs
 
-# see test_utilities.py for explanation
-shortest_h_com_file = "SXS:BBH:0156v1/Lev5/rhOverM_Asymptotic_GeometricUnits_CoM.h5"
+from .conftest import shortest_h_com_file
+
+try:
+    import spinsfast
+    requires_spinsfast = lambda f: f
+except:
+    requires_spinsfast = pytest.mark.skip(reason="spinsfast is missing")
 
 
 def test_backwards_compatibility():
@@ -30,10 +38,8 @@ def test_backwards_compatibility():
                     assert np.array_equal(f[group], h[group])
 
 
-@pytest.mark.skipif(not sys.platform=="linux", reason="Cannot install spinsfast on Windows; pip sucks on mac")
-def test_boost():
-    with contextlib.redirect_stdout(None):
-        h = sxs.load(shortest_h_com_file, extrapolation_order=3)
+@requires_spinsfast
+def test_boost(h):
     ell_max = 2*h.ell_max
     hprime = h.boost(np.array([0.01, 0.02, 0.03]), ell_max=ell_max)
     assert h.spin_weight == hprime.spin_weight
@@ -41,17 +47,15 @@ def test_boost():
     assert hprime.ell_max == ell_max
 
 
-@pytest.mark.skipif(sys.platform.startswith("win"), reason="Cannot install spherical_functions on Windows")
-@pytest.mark.xfail
 def test_modes_conjugate():
-    import spherical_functions as sf
-    tolerance = 1e-15
+    import spherical as sf
+    tolerance = 1e-14
     np.random.seed(1234)
     for inplace in [False, True]:
         for s in range(-2, 2 + 1):
             ell_min = abs(s)
             ell_max = 8
-            a = np.random.rand(3, 7, sf.LM_total_size(ell_min, ell_max)*2).view(complex)
+            a = np.random.rand(3, 7, sf.Ysize(ell_min, ell_max)*2).view(complex)
             m = sf.Modes(a, spin_weight=s, ell_min=ell_min, ell_max=ell_max)
             g = m.grid()
             s = m.s
@@ -64,20 +68,20 @@ def test_modes_conjugate():
             assert ell_min == mbar.ell_min
             assert ell_max == mbar.ell_max
             assert shape == mbar.shape
-            assert np.allclose(g, np.conjugate(gbar), rtol=tolerance, atol=tolerance)
+            assert np.allclose(g, np.conjugate(gbar), rtol=tolerance, atol=tolerance), (
+                f"max|g-ḡ.bar| = {np.max(np.abs(g - np.conjugate(gbar)))}"
+            )
 
 
-@pytest.mark.skipif(sys.platform.startswith("win"), reason="Cannot install spherical_functions on Windows")
-@pytest.mark.xfail
 def test_modes_real():
-    import spherical_functions as sf
+    import spherical as sf
     tolerance = 1e-14
     np.random.seed(1234)
     for inplace in [False, True]:
         s = 0
         ell_min = abs(s)
         ell_max = 8
-        a = np.random.rand(3, 7, sf.LM_total_size(ell_min, ell_max)*2).view(complex)
+        a = np.random.rand(3, 7, sf.Ysize(ell_min, ell_max)*2).view(complex)
         # Test success with spin_weight==0
         m = sf.Modes(a, spin_weight=s, ell_min=ell_min, ell_max=ell_max)
         g = m.grid()
@@ -101,17 +105,15 @@ def test_modes_real():
                 mreal = m._real_func(inplace)
 
 
-@pytest.mark.skipif(sys.platform.startswith("win"), reason="Cannot install spherical_functions on Windows")
-@pytest.mark.xfail
 def test_modes_imag():
-    import spherical_functions as sf
+    import spherical as sf
     tolerance = 1e-14
     np.random.seed(1234)
     for inplace in [False, True]:
         s = 0
         ell_min = abs(s)
         ell_max = 8
-        a = np.random.rand(3, 7, sf.LM_total_size(ell_min, ell_max)*2).view(complex)
+        a = np.random.rand(3, 7, sf.Ysize(ell_min, ell_max)*2).view(complex)
         # Test success with spin_weight==0
         m = sf.Modes(a, spin_weight=s, ell_min=ell_min, ell_max=ell_max)
         g = m.grid()
@@ -158,7 +160,7 @@ def test_modes_squared_angular_momenta():
     for s in range(-2, 2+1):
         ell_min = abs(s)
         ell_max = 8
-        a = np.random.rand(3, 7, sf.LM_total_size(ell_min, ell_max)*2).view(complex)
+        a = np.random.rand(3, 7, sf.Ysize(ell_min, ell_max)*2).view(complex)
         m = sf.Modes(a, spin_weight=s, ell_min=ell_min, ell_max=ell_max)
 
         # Test L^2 = 0.5(L+L- + L-L+) + LzLz
@@ -198,7 +200,7 @@ def test_modes_derivative_commutators():
     for s in range(-2, 2+1):
         ell_min = abs(s)
         ell_max = 8
-        a = np.random.rand(3, 7, sf.LM_total_size(ell_min, ell_max)*2).view(complex)
+        a = np.random.rand(3, 7, sf.Ysize(ell_min, ell_max)*2).view(complex)
         m = sf.Modes(a, spin_weight=s, ell_min=ell_min, ell_max=ell_max)
         # Test [Ri, Lj] = 0
         for R in [Rz, Rp, Rm]:
@@ -224,3 +226,71 @@ def test_modes_derivative_commutators():
         assert np.allclose(Rp(Rm(m)) - Rm(Rp(m)), 2 * Rz(m), rtol=tolerance, atol=tolerance)
         # Test [ethbar, eth] = 2s
         assert np.allclose(ethbar(eth(m)) - eth(ethbar(m)), 2 * m.s * m, rtol=tolerance, atol=tolerance)
+
+
+def test_modes_evaluate(h, eps):
+    import time
+
+    ell_max = h.ell_max
+    ϵ = 5 * (2 * ell_max + 1) * 2 * eps
+
+    m1 = h[:1000].copy()
+    m2 = h[:1000].copy()
+    m2._metadata["frame_type"] = "corotating"
+    m2._metadata["frame"] = quaternionic.one * np.ones_like(h.t)
+
+    equiangular_grid = spherical.theta_phi(2 * ell_max + 1, 2 * ell_max + 1)
+
+    t0 = time.perf_counter()
+    g1 = m1.evaluate(equiangular_grid)
+    t1 = time.perf_counter()
+    g2 = m2.evaluate(equiangular_grid)
+    t2 = time.perf_counter()
+
+    print()
+    print(f"\tTime for inertial frame: {t1-t0:.4f} seconds")
+    print(f"\tTime for 'rotating' frame: {t2-t1:.4f} seconds")
+
+    assert np.allclose(g1, g2, rtol=ϵ, atol=ϵ), f"max|g1-g2|={np.max(np.abs(g1-g2))}"
+
+
+def test_modes_rotate(h, eps):
+    import time
+
+    ϵ = 5 * (2 * h.ell_max + 1) * 2 * eps
+
+    print()
+    for i, R in enumerate([quaternionic.one, quaternionic.one * np.ones_like(h.t)]):
+        t1 = time.perf_counter()
+        hprm = h.rotate(R)
+        t2 = time.perf_counter()
+        print(f"\tRotation {i+1} took {t2-t1:.4f} seconds")
+        assert type(h) == type(hprm)
+        assert np.array_equal(h.t, hprm.t)
+        assert np.allclose(h.ndarray, hprm.ndarray, rtol=ϵ, atol=ϵ)
+
+        metadata = h._metadata.copy()
+        metadataprm = hprm._metadata.copy()
+        for d in [metadata, metadataprm]:
+            for key in ['time', 'frame']:
+                d.pop(key, None)
+            for key in ['space_translation', 'boost_velocity']:
+                d[key] = d[key].tolist()
+        assert metadata == metadataprm
+
+
+def test_modes_rotate_evaluate(h, Rs, eps):
+    """Test that evaluating modes at rotated points == evaluating rotated modes at points"""
+    import time
+
+    ell_max = h.ell_max
+    ϵ = (2 * h.ell_max + 1) * 2 * eps
+
+    equiangular_grid = spherical.theta_phi(2 * ell_max + 1, 2 * ell_max + 1)
+    Rθϕ = quaternionic.array.from_spherical_coordinates(equiangular_grid)
+
+    for i, R in enumerate(Rs):
+        hprm = h.copy().rotate(R)  # hprm = h @ 𝔇(R)
+        m1 = hprm.evaluate(Rθϕ)  # m1 = hprm @ 𝔇(Rθϕ) √...
+        m2 = h.evaluate(R * Rθϕ)  # m2 = h @ 𝔇(R * Rθϕ) √...
+        assert np.allclose(m1, m2, rtol=ϵ, atol=ϵ)
